
/*

  GAI-Pal 0.7 by Jonas Aaberg <cja@gmx.net> 2002-2003, inspiration and 
  graphics by A.Sleep and Hadess wmpal applet.

  I think the images are originally made by artwiz@themes.org for bbpal.

  Released under GNU GPL

*/

#include "config.h"
#include <stdio.h>
#include <gai/gai.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#ifdef XOSD
#include <xosd.h>
#endif

#define PALS 7
#define DEFAULT_PAL 3

static GdkPixbuf *pal_image[3];

static char *fortune_prg;
static int on_start_up;
static int first_time = 1;
static int force_update = 0, real_start_up;

static GtkWidget *fortune_window=NULL;

#ifdef XOSD
static char *display_names[] = {"In a window", "On screen display", NULL};
static int display_val = 0;

#define XOSD_RED 0
#define XOSD_GREEN 1
#define XOSD_BLUE 2
#define XOSD_WHITE 3
#define XOSD_BLACK 4

#define XOSD_SMALL 0
#define XOSD_MEDIUM 1
#define XOSD_LARGE 2

static char *text_size_names[] = {"Small", "Medium", "Large", NULL}, *text_color_names[] = {"Red", "Green", "Blue", "White", "Black", NULL};
static int text_size = XOSD_MEDIUM, text_color = XOSD_GREEN, text_delay = 2;
static GaiSS text_limits = {0, 60, 1}; 

static xosd *osd;
static int osd_alive = FALSE;
#endif

static int pal_val = 0;
static char *nice_names[] = {"Demon", "Dino", "Grape", "Pear", "Sun", "Turtle", "Calvin" ,NULL};
static GaiPI pal_pref[] = {
#ifdef XOSD
    {GAI_NOTEBOOK, "General"},
#endif
    {GAI_CHECKBUTTON, "Show fortune cookie on start up", &on_start_up, &on_start_up},
    {GAI_TEXTENTRY,"Fortune program including command line options:", &fortune_prg, &fortune_prg},
    {GAI_FRAME, "Choose a friend:"},
    {GAI_RADIOBUTTON, nice_names, &pal_val, &pal_val},
    {GAI_FRAME_E},

#ifdef XOSD

    {GAI_FRAME, "Display fortune"},
    {GAI_RADIOBUTTON, display_names, &display_val, &display_val},
    {GAI_FRAME_E},
    {GAI_NOTEBOOK_E},

    {GAI_NOTEBOOK, "OSD"},
    {GAI_SPINBUTTON, "Text display delay", &text_delay, &text_delay, &text_limits},
    {GAI_FRAME, "Text size"},
    {GAI_RADIOBUTTON, text_size_names, &text_size, &text_size},
    {GAI_FRAME_E},
    {GAI_FRAME, "Text color"},
    {GAI_RADIOBUTTON, text_color_names, &text_color, &text_color},
    {GAI_FRAME_E},
    {GAI_NOTEBOOK_E},
#endif
    {GAI_END}};



void fortune_die(void)
{
    gtk_widget_destroy(fortune_window);
    fortune_window=NULL;
} 

void pal_window(char *msg)
{
  int w,h;

  if(fortune_window!=NULL){
      gtk_window_present(GTK_WINDOW(fortune_window));
      return;
  }



  fortune_window = gtk_message_dialog_new (NULL,0,GTK_MESSAGE_INFO,GTK_BUTTONS_OK,msg);
  gtk_widget_realize(fortune_window);
  gdk_window_get_geometry(fortune_window->window,NULL,NULL,&w,&h,NULL);
  gtk_window_set_position(GTK_WINDOW(fortune_window),GTK_WIN_POS_CENTER);

  gtk_widget_show_all (fortune_window);

  g_signal_connect_swapped (G_OBJECT (fortune_window), "response",
			    G_CALLBACK (fortune_die), NULL);


}
#ifdef XOSD
void pal_xosd(char *msg)
{


    int msg_ptr[200];
    int lines = 0,i,  len;

    if(osd_alive){
	if(xosd_is_onscreen(osd))
	    return;
    }
    msg_ptr[0] = 0;

    len = strlen(msg);
    if (!len)
      return;

    for(i=0;i<len;i++){
	if(msg[i] == '\n'){
	    lines++;
	    if(lines == 200)
		break;
	    msg_ptr[lines] = i+1;
	    msg[i] = '\0';
	}
	if(msg[i] == '\t')
	    msg[i] = ' ';
    }
    
    osd = xosd_create(lines);

    switch(text_size){
    case XOSD_SMALL:
	xosd_set_font(osd, "-*-fixed-*-*-*-*-10");
	xosd_set_vertical_offset(osd, gdk_screen_height()/2-10*lines);
	break;
    case XOSD_MEDIUM:
	xosd_set_font(osd, "-*-fixed-*-*-*-*-18");
	xosd_set_vertical_offset(osd, gdk_screen_height()/2-18*lines);
	break;
    case XOSD_LARGE:
	xosd_set_font(osd, "-*-fixed-*-*-*-*-24");
	xosd_set_vertical_offset(osd, gdk_screen_height()/2-24*lines);
	break;
    }
    switch(text_color){
    case XOSD_RED:
	xosd_set_colour(osd, "red");
	break;
    case XOSD_GREEN:
	xosd_set_colour(osd, "LawnGreen");
	break;
    case XOSD_BLUE:
	xosd_set_colour(osd, "SteelBlue");
	break;
    case XOSD_WHITE:
	xosd_set_colour(osd, "snow");
	break;
    case XOSD_BLACK:
	xosd_set_colour(osd, "black");
	break;

    }


    xosd_set_align(osd, XOSD_center);

    xosd_set_timeout(osd, lines+text_delay);
    xosd_set_shadow_offset(osd, 1);

    for(i=0;i<lines;i++)
    	xosd_display(osd, i, XOSD_string, msg + msg_ptr[i]);

    osd_alive = TRUE;

}

#endif

void pal_click(int x, int y, gpointer d)
{
    char *msg;
    const char errorMsg[] = "Fortune program not found!\n";
    GError *err = NULL;

    if ( !g_spawn_command_line_sync (fortune_prg, &msg, NULL, NULL, &err) )
    {
        g_error_free( err );
        msg = g_strdup( errorMsg );
    }
#ifdef XOSD
    if(display_val == 0)
	pal_window(msg);
    else
	pal_xosd(msg);
#else
    pal_window(msg);
#endif
    g_free(msg);

}


void update_pal(gpointer d)
{

    int i;
#ifdef XOSD
    if(!xosd_is_onscreen(osd) && osd_alive){
	xosd_destroy(osd);
	osd_alive = FALSE;
    }
#endif
    
    if(pal_image[0]==NULL) return;

    if(first_time){
	first_time=0;

	i=rand()%3;
	if(on_start_up && real_start_up)
	    pal_click(0,0,NULL);
	real_start_up = 0;

    }
    else
	i=rand()%10;

    if(force_update){
	i=rand()%3;
	force_update=0;
    }

    if(i<3){
	gai_draw_bg(pal_image[i],0,0,
		gdk_pixbuf_get_width(pal_image[i]),gdk_pixbuf_get_height(pal_image[i]),4,4);
	gai_draw_update_bg();
    }

}
void pal_load_images(void)
{
    char *pal_file;
    char *names[]={"demon","dino","grape","pear","sun","turtle", "calvin"};
    int i;

    for(i=1;i<4;i++){
	pal_file = g_strdup_printf("%s%d.png",names[pal_val],i);
	pal_image[i-1] = gai_load_image(pal_file);
	g_free(pal_file);
    }


}

void restart_pal(gpointer d)
{
    gai_save_int("pal/default_pal", pal_val);
    gai_save_int("pal/on_start_up", on_start_up);
#ifdef XOSD
    gai_save_int("pal/display_val", display_val);
    gai_save_int("pal/text_size", text_size);
    gai_save_int("pal/text_color", text_color);
    gai_save_int("pal/text_delay", text_delay);
#endif
    gai_save_string("pal/fortune_prg", fortune_prg);

    g_object_unref(pal_image[0]);
    pal_image[0]=NULL;

    g_object_unref(pal_image[1]);
    g_object_unref(pal_image[2]);

    pal_load_images();

    /* Make sure things are updated */
    first_time = 1;
    update_pal(NULL);
}


void pal_change(int orient, int width, int height, gpointer d)
{
    force_update = 1;
}


int main(int argc, char **argv)
{
    gai_init2(&applet_defines, &argc, &argv);

    srand(time(NULL));
    gai_background_set(64, 64, GAI_BACKGROUND_MAX_SIZE_NONE, TRUE);

    pal_val = gai_load_int_with_default("pal/default_pal",DEFAULT_PAL);
    on_start_up = gai_load_int_with_default("pal/on_start_up",0);
    fortune_prg = gai_load_string_with_default("pal/fortune_prg", "fortune");
#ifdef XOSD
    display_val = gai_load_int_with_default("pal/display_val", 1);
    text_size = gai_load_int_with_default("pal/text_size", XOSD_MEDIUM);
    text_color = gai_load_int_with_default("pal/text_color", XOSD_GREEN);
    text_delay = gai_load_int_with_default("pal/text_delay", 2);
#endif

    real_start_up = 1;

    pal_load_images();

    gai_tooltip_set("I'm your pal :-)");
    gai_signal_on_update((GaiCallback0 *)update_pal, 1000, NULL);
    gai_signal_on_mouse_button_click((GaiCallback2 *)pal_click, GAI_MOUSE_BUTTON_1, NULL);
    gai_signal_on_change((GaiCallback3 *)pal_change, NULL);

    gai_preferences2("GAI Pal - Preferences", pal_pref, "Don't even think about that! ;-)", (GaiCallback0 *)restart_pal, NULL);
    
    gai_start();
    return 0;    
}
